(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     29651,        958]*)
(*NotebookOutlinePosition[     32755,       1070]*)
(*  CellTagsIndexPosition[     32332,       1050]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\[Copyright] 1997-2002 K. Sutner ", "SmallText"],

Cell[TextData[StyleBox["Regular Expressions",
  FontFamily->"Charter"]], "Title",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:1"],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Regular\ Expressions\)], "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->{"Regular expressions", "c:2"}],

Cell["\<\
Regular expressions are a notation system for regular languages: \
for every regular language there is a regular expression that denotes it (in \
fact, there are infinitely many). Unlike finites state machines, regular \
expressions are concise and can be easily written out by a human being. More \
importantly, they can also be typed on a commandline, and thus can be used \
easily as input to programs.  Hence, regular expressions have become a \
standard tool to describe patterns of strings in such programs as editors or \
search algorithms. 
The next section describes regular expressions in automata.  For \
applications, the most important feature of regular expressions is that there \
are fast algorithms to convert them to corresponding finite state machines. \
We will present two such algorithms. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Representation\ of\ Regular\ Expressions\)], \
"Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:3"],

Cell[TextData[{
  "Regular expressions describe languages composed of trivial basic \
components suxh as single characters using the operations of union, \
concatenation and Kleene (repetition).  In automata, they are represented \
using special headers:\n\tREP\t\tfor  plus, union, \n\tRET\t\tfor  times, \
concatenation,  and\n\tRES\t\tfor Kleene star (repetition).\nRES is unary, \
REP and RET are polyadic (like Plus and Times in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ").  Also, Empty stands for the empty set and Eps for  {Eps}.  Symbol s \
denotes the singleton language {s}.  Thus, regular expressions are defined by \
the following context free grammar (cum grano salis):\n\n\tE \[Rule] Empty  | \
 Eps  |  symbol\n\tE \[Rule]  REP[ E,..,E]  |  RET[ E,..,E]   |  RES[E].\n\n\
The print format of regular expressions uses Plus, Times and a '*' to \
represent REP, RET and RES, respectively. If you want to be absolutely \
certain what form a regular expression takes, it is best to look at its \
FullForm. \n\t\t\t\t\t\t\t\t\nTo facilitate the input of regular expressions, \
the user can write these expressions also in algebraic notation, very much \
like ordinary arithmetic. The symbols of the underlying alphabet are \
variables and operators are Plus, NonCommutativeMultiply and RES. For \
example, we can write \n\n\ta ** RES[a+b] ** bb\t instead of\t\
RET[a,RES[REP[a,b]],b,b]. \n\nThe command ToRegExp transforms such a term \
into a proper regular expression."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(re = Eps + RES[a + b] ** b\), "\n", 
    \(ToRegExp[re]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Note that the star operator prints in a funny way:  it is attached \
to its argument in postfix notation, then the whole expression is enclosed in \
parentheses.  This can be rather confusion on occasion, and can be avoided by \
either not using the arithmetic abbreviations or by applying ToRegExp:\
\>", \
"Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(RES[a + b]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "One of the major obstacles in dealing with regular expressions is the lack \
of a good normal form. For example, the next expression clearly could be \
simplified. A collection of simplification rules is available under the name \
",
  StyleBox["$regexrules",
    FontFamily->"Chicago"],
  ". Since there is no finite basis for the equations of the corresponding \
closed semiring the collection is by necessity lacking. The user should feel \
free to augment it. Make sure not to introduce infinite loops, though. \nHere \
is an (the?) example where the simplification rules work properly."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(REP[\ RET[b, RES[b]], \ Eps\ ]\), "\n", 
    \(% //. \[InvisibleSpace]$regexrules\)}], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Conversion\ RE\ to\ FSMs\)], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:4"],

Cell[TextData[{
  "For the conversion of regular expressions to finite state machines it is \
best to use a special normal form finite state machine. The normal form \
should make it easy to perform the operations union, conacatenation and star. \
There are two normal forms implemented in automata: Thompson style begin/exit \
automata and McNaughton-Yamada style normalized NFAs (or rather, their \
tails). We discuss both types below. \nA begin-exit FA (or BEFA) is a finite \
automaton that has exactly one initial and one final state.  Also, the \
initial state has indegree zero and the final state has outdegree zero (in \
the transition diagram of the automaton).  This makes it easy to compute \
union, concatenation and star of BEFAs. In this package, a BEFA is  described \
by an expression of the form \t\n\t\tBEFA[n,T].\nAlternatively one can use \
tails of normalized NFAs. A normalized NFA (NNFA) has exactly one initial \
state of indegree 0 and all transitions leading to the same state carry the \
same label. There are no epsilon transitions. It is easy to see that such a \
machine can be reconstructed from the following pieces: \n\t- ",
  Cell[BoxData[
      \(TraditionalForm\`Q\  - \ {q\_0}\)]],
  ", the states other than the initial state,\n\t-  the unlabeled edges of \
the transition diagram,\n\t-  the labels associated with each vertex other \
than ",
  Cell[BoxData[
      \(TraditionalForm\`q\_0\)]],
  ",\n\t-  the states at distance 1 from ",
  Cell[BoxData[
      \(TraditionalForm\`q\_0\)]],
  ",\n\t-  the final states,\n\t-  a bit indicating whether the machine \
accepts \[Epsilon]. \nThese six pieces are the tail of the automaton. Again,  \
it is easy to compute union, concatenation and star of tails. In this package \
we use \n\t\tNNFA[ n, E, L, I, F, e ]\t\nto denote tails of NNFAs. \nBoth \
conversion procedures produce a finite state machine whose number of states \
is bounded by 2 times the length of the regular expression. The Thompson \
method produces a machine with epsilon moves in linear time. The \
McNaughton-Yamada algorithm directly produces an NFA, however, the number of \
transitions in the machine may not be linear in the length of the regular \
expression. Indeed, the implementation here is not optimized for speed, its \
main purpose is to show an alternative way of converting regular expressions. \
\n\nThe procedures BuildBEFA constructs a BEFA for the language {w} directly \
from input w (the underlying alphabet is irrelevant here). Similarly, \
BuildNNFA constructs a NNFA."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(BuildBEFA[Eps]\), "\n", 
    \(BuildNNFA[Eps]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(BuildBEFA[abba]\), "\n", 
    \(BuildNNFA[abba]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Note that since BEFAs have no separate field for the underlying \
alphabet, the transition list is given in full form. Also observe that \
conversion of the tail to an NFA to an FA adds one state, so the last two \
machines are really identical.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(ToFA[BuildBEFA[abba], {a, b}] === 
      ToFA[BuildNNFA[abba], {a, b}]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
To convert a regular expression to a BEFA or an NNFA use the \
commands\tRegularExpressionToBEFA and RegularExpressionToNNFA, respectively. \
To convert to an FA use RegularExpressionToFA. The last command takes as \
input a regular expression as well as an alphabet. There are several options \
associated with RegularExpressionToFA : \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Options[RegularExpressionToFA]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\

To obtain a BEFA  we have to set Thompson->True, otherwise an NNFA will be \
produced. If we wish to prevent epsilon-elimination we have to set \
EpsElim->False.  The last option SizeOnly causes the size of the resulting FA \
to be computed without actually constructing the machine.  \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(e = RES[aa + bb];\)\), "\n", 
    \(m1 = RegularExpressionToFA[e, {a, b}]\), "\n", 
    \(m2 = 
      RegularExpressionToFA[e, {a, b}, Thompson \[Rule] True, 
        EpsElim \[Rule] False]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Thus, the NNFA is bigger than the BEFA in this case. A few other \
examples comparing the size of the two alternative automata are given below. \
Of course, the machines are equivalent. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(EquivalentQFA[m1, m2]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
The trace of EpsilonEliminationFA shows the the epsilon and \
non-epsilon transitions, and the transitive closure of the epsilon \
transitions in m2. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\({mm2, treps, trnoneps, transclos} = 
        EpsilonEliminationFA[m2, DoTrace \[Rule] True];\)\), "\n", 
    \(treps\), "\n", 
    \(trnoneps\), "\n", 
    \(EnumForm[transclos]\)}], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Examples\)], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:5"],

Cell[CellGroupData[{

Cell["\<\
  Example 1: 
  Any two consecutive b's are separated by an even number of a's.\
\>", \
"Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:6"],

Cell["\<\
There is a straightforward regular expression for this \
language:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["exp = ToRegExp[  RES[a] ** RES[ b ** RES[aa] ] ** RES[a] ]", "Input"],

Cell[BoxData[{
    \(\(m1 = RegularExpressionToFA[exp, {a, b}];\)\), "\n", 
    \(\(mm1 = MinimizeFA[m1];\)\), "\n", 
    \(m1\), "\n", 
    \(mm1\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
It is interesting to look at the rewrite system for the syntactic \
semigroup of this language: any even number of a's is equivalent to none, any \
positive number of b's is equivalent to a single b, and certain bad patterns \
must not occur.  \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\({S, wit, eq} = SyntacticSG[mm1, Equations \[Rule] True];\)\), "\n", 
    \(TableForm[PrintEquations[eq]]\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "The reductions given by the above equations transform words in L6 (and \
indeed in the whole language  L = ",
  StyleBox["a*(b (aa)*)*a* )",
    FontFamily->"Courier"],
  " as follows (see the section on syntactic semigroups below  for more \
details):"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(L6 = LanguageFA[mm1, 6]\), "\n", 
    \(\($identitystring = "\<Eps\>";\)\), "\n", 
    \(\(red = Union[\((WordReduce[#1, eq] &)\) /@ L6];\)\), "\n", 
    \(WordSort[red]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
The equations, while correct, are not in the form one might expect. \
They can be rewritten as follows:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\($identitystring = "\<1\>";\)\), "\n", 
    \(\(eqs = SimplifyEquations[S, wit, eq];\)\), "\n", 
    \(TableForm[PrintEquations[eqs]]\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "The one bad pattern which forces a string out of the language is ",
  StyleBox["bab",
    FontSlant->"Italic"],
  " (remember the other simplification rules). "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["  Example 2: Even number of a's and b's.", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "A regular expression for all strings with an even number of a's and b's is \
quite a bit harder to come up with than the corresponding DFA.  Clearly, the \
expression will be a star.  It helps to think of the expression as a \
describing a sequence of blocks in the string.  The easy case is when the \
string starts with ",
  StyleBox["aa",
    FontSlant->"Italic"],
  " or ",
  StyleBox["bb",
    FontSlant->"Italic"],
  ".  The other cases are handled as follows:"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
e0 = aa + bb;
e1 = ab + ba;
exp = RES[ e0 + e1**RES[e0]**e1 ]\
\>", "Input"],

Cell["\<\
We can first check how big the FA will be without actually \
constructing it:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(RegularExpressionToFA[exp, {a, b}, SizeOnly \[Rule] True]\)], "Input",
  AspectRatioFixed->True],

Cell["No problem. Now we actually build the machine:", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(m1 = RegularExpressionToFA[exp, {a, b}]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Using the epsilon-transition based construction we get a slightly \
smaller machine:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(m2 = 
        RegularExpressionToFA[exp, {a, b}, 
          Thompson \[Rule] True];\)\), "\n", 
    \(Size[m2]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Of course, it is much easier to get the same DFA from an \
intersection operation.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(m3 = 
      IntersectionFA[ModCounterDFA[a, 0, 2, {a, b}], 
        ModCounterDFA[b, 0, 2, {a, b}]]\), "\n", 
    \(IsomorphicQDFA[MinimizeFA[m2], m3]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
It is a very good exercise to construct an expression for the \
language with an odd number of a's and b's.  Make sure to verify your \
expression by converting it to an FA and comparing the corresponding minimal \
DFA to the obvious one. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["  Example 3: L = a* b* + bab. ", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:8"],

Cell["exp = RES[a]**RES[b] + bab", "Input"],

Cell[BoxData[{
    \(\(m = RegularExpressionToFA[exp, {a, b}];\)\), "\n", 
    \(\(mm = MinimizeFA[m];\)\), "\n", 
    \(PrintFA[mm]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Machine mm has a sink, but is not a sink automaton (there is a \
non-final state other than the trap).\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(SinkQDFA[mm]\), "\n", 
    \(TheSinkDFA[mm]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(ColumnForm[LanguageFA[mm, \(-6\)]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(ListPlot[LanguageFA[mm, \(-20\), SizeOnly \[Rule] True], 
        PlotStyle \[Rule] {Blue, PointSize[0.02]}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["  Example 4: \"no four consecutive a's\". ", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:9"],

Cell["\<\
The hardest part here is to come up with the right regular \
expression. Of course, everything would be easy if we had extended regular \
expressions that have a complementation operation built in.  Since we don't \
(good question: why not?), one has to be a little careful about decomposing \
strings into pieces. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["exp = (Eps + a + aa + aaa ) ** RES[b + ba + baa + baaa]", "Input"],

Cell[BoxData[
    \(m0 = ToDFA[RegularExpressionToFA[exp, {a, b}]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(m1 = MinimizeFA[m0]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Verify that m1 works properly, first empirically and then by \
constructing the minimal automaton in a different way. The two machines turn \
out to be identical.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Complement[Words[6, {a, b}], LanguageFA[m1, 6]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(AlgebraicForm /@ %\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(MinimizeFA[ComplementFA[InfixFA[aaaa, {a, b}]]]\), "\n", 
    \(m1 === %\)}], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["  Example 5: \"integers in base 5\"", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:10"],

Cell["\<\
It is a standard exercise to show that the set of numbers in some \
notation system forms a regular language. Here is a simple example: integers \
in base 5, without leading zeros. First we set up the alphabet--digits and a \
unary minus--and some abbreviations:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(alph = ToString /@ {0, 1, 2, 3, 4, "\<-\>"}\), "\n", 
    \(digs0 = REP @@ Take[alph, 5]\), "\n", 
    \(digs = Rest[digs0]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Then one constructs a regular expression and the corrsponding \
machines.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
reg = (\"-\" + Eps) ** digs ** RES[digs0] + \"0\"
m=RegularExpressionToFA[reg,alph]
mm=MinimizeFA[m]\
\>", "Input"],

Cell["Lastly, we check the language:", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(ColumnForm[LanguageFA[mm, \(-3\)]]\)], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["  Example 6: Equivalence ", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:11"],

Cell["\<\
Testing whether two regular expressions are equivalent is \
PSPACE-complete in general, so one should not try the following for large \
expressions.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
e1 = RES[aa ** RES[ba]]
e2 = a ** RES[aa+ab] ** a + Eps\
\>", "Input"],

Cell[BoxData[{
    \(\(m1 = RegularExpressionToFA[e1, {a, b}];\)\), "\n", 
    \(\(m2 = RegularExpressionToFA[e2, {a, b}];\)\), "\n", 
    \(EquivalentQFA[m1, m2]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
Here is another example, which actually represents one of the \
fundamental valid equation of regular expressions:
\t(ab)* = a(ba)*b + eps.
Hera a and b could be arbitrary regular expressions, but it suffices to \
verify the equation for atomic symbols. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
e1 = RES[ab]
e2 = a**RES[ba]**b + Eps\
\>", "Input"],

Cell[BoxData[{
    \(\(m1 = RegularExpressionToFA[e1, {a, b}];\)\), "\n", 
    \(\(m2 = RegularExpressionToFA[e2, {a, b}];\)\), "\n", 
    \(EquivalentQFA[m1, m2]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
One more fundamental equation:
\t(a+b)* = (a* b*)*.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(e1 = RES[a + b]\), "\n", 
    \(e2 = RES[\ RES[a]\  ** \ RES[b]\ ]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(m1 = RegularExpressionToFA[e1, {a, b}];\)\), "\n", 
    \(\(m2 = RegularExpressionToFA[e2, {a, b}];\)\), "\n", 
    \(EquivalentQFA[m1, m2]\)}], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Equations\ of\ Regular\ Expressions\)], "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:12"],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Conversion\ to\ Regular\ Expressions\)], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:13"],

Cell["\<\
There is a theorem by Kleene according to which every regular \
language can be denoted by some regular expression. The proof shows how to \
construct the expression from a given machine for the language and is mostly \
of academic interest: in applications one usually has the regular expression \
and wants to construct a corresponding machine. One of the problems with the \
algorithm is that the resulting expressions are usually huge (see the \
discussion about simplification of regular expressions above). Therefore, one \
should not use the command ToRegularExpressionFA on machines with more than a \
few states. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(m = 
      IntersectionFA[ModCounterDFA[a, 0, 2, {a, b}], 
        ModCounterDFA[b, 0, 2, {a, b}]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(exp = ToRegularExpressionFA[m]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Since the expression is quite clumsy, the corresponding machine \
obtained by converting back to an FA will be fairly large. However, it is \
equivalent to the original DFA.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(mm = RegularExpressionToFA[exp, {a, b}]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
This FA is trim, but the corresponding DFA has only 29 states. \
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Size[TrimFA[mm]]\), "\n", 
    \(ToDFA[mm]\), "\n", 
    \(EquivalentQFA[m, mm]\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
As a second example, construct a machine for all words not \
containing an infix `aba'. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(m = AccessibleFA[MinimizeFA[ComplementFA[InfixFA[aba, 2]]]]\)], "Input",\

  AspectRatioFixed->True],

Cell["A horrendous rational expression for this language:", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(re = ToRegularExpressionFA[m]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
To make sure this is right, convert back and minimize.  Note that \
the intermediate NFA has 54 states!  In the end, the minimal DFA mm is \
isomorphic to m, so the expression is correct.
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(nfa = RegularExpressionToFA[re];\)\), "\n", 
    \(Size[nfa]\), "\n", 
    \(dfa = ToDFA[nfa]\), "\n", 
    \(mm = MinimizeFA[dfa]\), "\n", 
    \(IsomorphicQDFA[m, mm]\)}], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm\`Machines\ as\ Systems\ of\ Equations\)], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:14"],

Cell["\<\
One may view a FSM as a linear system of equations over the \
semiring of all languages over some alphabet. We have a command \
DFAToEquations that converts a DFA to the corresponding system of equations. \
The system can the be solved using Arden's lemma. As an example we show how \
to get a regular expression for the language of all strings with an even \
number of a's and b's.  

As already pointed out on several occasions, simplification of regular \
expressions is difficult (an understatement) Hence this approach will only be \
useful for very small machines at best. In other words: use only for \
carefully chosen examples and adjust the simplification rules whenever \
necessary. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Clear[X]\), "\[IndentingNewLine]", 
    \(\(m = 
        IntersectionFA[ModCounterDFA[a, 0, 2, {a, b}], 
          ModCounterDFA[b, 0, 2, {a, b}]];\)\), "\n", 
    \(\({e1, e2, e3, e4} = ToEquationsDFA[m];\)\), "\n", 
    \(ColumnForm[{e1, e2, e3, e4}]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e11 = Substitute[e1, e2, e3]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e44 = Substitute[e4, e2, e3]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e5 = Arden[e44]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e6 = Substitute[e11, e5]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(sol = Arden[e6]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
The last expression could be simplified by rewriting the four \
middle terms (but our simplification rules are not powerful enough to do \
this). At any rate, we can verify that the expression is correct by \
converting it back to a DFA.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(exp = sol\[LeftDoubleBracket]2\[RightDoubleBracket];\)\), "\n", 
    \(ColumnForm[
      ToList[exp\[LeftDoubleBracket]1\[RightDoubleBracket]]]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(mm = MinimizeFA[RegularExpressionToFA[exp, {a, b}]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(IsomorphicQDFA[m, mm]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\

Another example: all words containing the factor  `aba'.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(m = AccessibleFA[MinimizeFA[InfixFA[aba, 2]]]\), "\n", 
    \(\({e1, e2, e3, e4} = ToEquationsDFA[m];\)\), "\n", 
    \(ColumnForm[{e1, e2, e3, e4}]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e44 = Arden[e4]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e33 = Substitute[e3, e44]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e21 = Substitute[e2, e33]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e22 = Arden[e21]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e11 = Substitute[e1, e22]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(sol = Arden[e11]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(MinimizeFA[
      RegularExpressionToFA[
        sol\[LeftDoubleBracket]2\[RightDoubleBracket]]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(IsomorphicQDFA[%, m]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Of course, there is a much simpler rational expression for this \
language and we can find it without having to convert any machines:
\t(a+b)*aba(a+b)*

It is much harder to produce an expression for the complement of this \
language.  However, the same sequence of substitution and applications of \
Arden's lemma will generate a suitable expression when applied to the \
complement of the DFA from the last example: \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(mm = ComplementFA[m]\), "\n", 
    \(\({e1, e2, e3, e4} = ToEquationsDFA[mm];\)\), "\n", 
    \(ColumnForm[{e1, e2, e3, e4}]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e44 = Arden[e4]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e33 = Substitute[e3, e44]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e21 = Substitute[e2, e33]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e22 = Arden[e21]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e11 = Substitute[e1, e22]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(sol = Arden[e11]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(MinimizeFA[
      RegularExpressionToFA[
        sol\[LeftDoubleBracket]2\[RightDoubleBracket]]]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "The expression can be rewritten as \n\t",
  Cell[BoxData[
      \(TraditionalForm\`\((\((\[Epsilon]\  + \ a\[VeryThinSpace]a*b)\) b)\)*
        a*\((\[Epsilon]\  + \ a\[VeryThinSpace]b)\)\)]],
  "\nOf course, our simplification rules are not powerful enough to do this. \
"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.2 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{970, 992},
WindowMargins->{{Automatic, 109}, {Automatic, 0}},
PrintingStartingPageNumber->64,
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->True,
CellLabelAutoDelete->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
CharacterEncoding->Automatic,
Magnification->1.5,
StyleDefinitions -> "Demo.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:1"->{
    Cell[1762, 52, 148, 4, 88, "Title",
      Evaluatable->False,
      CellTags->"c:1"]},
  "Regular expressions"->{
    Cell[1935, 60, 164, 4, 115, "Section",
      Evaluatable->False,
      CellTags->{"Regular expressions", "c:2"}]},
  "c:2"->{
    Cell[1935, 60, 164, 4, 115, "Section",
      Evaluatable->False,
      CellTags->{"Regular expressions", "c:2"}]},
  "c:3"->{
    Cell[3010, 85, 164, 5, 57, "Subsection",
      Evaluatable->False,
      CellTags->"c:3"]},
  "c:4"->{
    Cell[6155, 161, 146, 4, 57, "Subsection",
      Evaluatable->False,
      CellTags->"c:4"]},
  "c:5"->{
    Cell[11555, 298, 130, 4, 57, "Subsection",
      Evaluatable->False,
      CellTags->"c:5"]},
  "c:6"->{
    Cell[11710, 306, 181, 7, 98, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:6"]},
  "c:7"->{
    Cell[14011, 388, 132, 3, 64, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:7"]},
  "c:8"->{
    Cell[16323, 474, 122, 3, 64, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:8"]},
  "c:9"->{
    Cell[17232, 510, 134, 3, 64, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:9"]},
  "c:10"->{
    Cell[18619, 559, 128, 3, 64, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:10"]},
  "c:11"->{
    Cell[19777, 603, 118, 3, 64, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:11"]},
  "c:12"->{
    Cell[21527, 671, 155, 4, 66, "Section",
      Evaluatable->False,
      CellTags->"c:12"]},
  "c:13"->{
    Cell[21707, 679, 159, 4, 57, "Subsection",
      Evaluatable->False,
      CellTags->"c:13"]},
  "c:14"->{
    Cell[24476, 774, 159, 4, 46, "Subsection",
      Evaluatable->False,
      CellTags->"c:14"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:1", 30610, 986},
  {"Regular expressions", 30728, 990},
  {"c:2", 30858, 994},
  {"c:3", 30988, 998},
  {"c:4", 31095, 1002},
  {"c:5", 31203, 1006},
  {"c:6", 31312, 1010},
  {"c:7", 31424, 1014},
  {"c:8", 31536, 1018},
  {"c:9", 31648, 1022},
  {"c:10", 31761, 1026},
  {"c:11", 31875, 1030},
  {"c:12", 31989, 1034},
  {"c:13", 32097, 1038},
  {"c:14", 32208, 1042}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1705, 50, 54, 0, 40, "SmallText"],
Cell[1762, 52, 148, 4, 88, "Title",
  Evaluatable->False,
  CellTags->"c:1"],

Cell[CellGroupData[{
Cell[1935, 60, 164, 4, 115, "Section",
  Evaluatable->False,
  CellTags->{"Regular expressions", "c:2"}],
Cell[2102, 66, 883, 15, 186, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3010, 85, 164, 5, 57, "Subsection",
  Evaluatable->False,
  CellTags->"c:3"],
Cell[3177, 92, 1556, 25, 658, "Text",
  Evaluatable->False],
Cell[4736, 119, 116, 3, 68, "Input"],
Cell[4855, 124, 373, 8, 79, "Text",
  Evaluatable->False],
Cell[5231, 134, 69, 2, 45, "Input"],
Cell[5303, 138, 670, 13, 150, "Text",
  Evaluatable->False],
Cell[5976, 153, 142, 3, 68, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6155, 161, 146, 4, 57, "Subsection",
  Evaluatable->False,
  CellTags->"c:4"],
Cell[6304, 167, 2611, 42, 791, "Text",
  Evaluatable->False],
Cell[8918, 211, 106, 3, 68, "Input"],
Cell[9027, 216, 108, 3, 68, "Input"],
Cell[9138, 221, 317, 7, 79, "Text",
  Evaluatable->False],
Cell[9458, 230, 129, 3, 45, "Input"],
Cell[9590, 235, 408, 8, 98, "Text",
  Evaluatable->False],
Cell[10001, 245, 89, 2, 45, "Input"],
Cell[10093, 249, 358, 8, 110, "Text",
  Evaluatable->False],
Cell[10454, 259, 252, 6, 91, "Input"],
Cell[10709, 267, 258, 6, 60, "Text",
  Evaluatable->False],
Cell[10970, 275, 80, 2, 45, "Input"],
Cell[11053, 279, 222, 6, 60, "Text",
  Evaluatable->False],
Cell[11278, 287, 240, 6, 137, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11555, 298, 130, 4, 57, "Subsection",
  Evaluatable->False,
  CellTags->"c:5"],

Cell[CellGroupData[{
Cell[11710, 306, 181, 7, 98, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:6"],
Cell[11894, 315, 138, 5, 41, "Text",
  Evaluatable->False],
Cell[12035, 322, 75, 0, 44, "Input"],
Cell[12113, 324, 186, 5, 114, "Input"],
Cell[12302, 331, 316, 7, 79, "Text",
  Evaluatable->False],
Cell[12621, 340, 167, 3, 68, "Input"],
Cell[12791, 345, 332, 9, 60, "Text",
  Evaluatable->False],
Cell[13126, 356, 230, 5, 114, "Input"],
Cell[13359, 363, 175, 5, 41, "Text",
  Evaluatable->False],
Cell[13537, 370, 195, 4, 91, "Input"],
Cell[13735, 376, 239, 7, 60, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[14011, 388, 132, 3, 64, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[14146, 393, 546, 14, 98, "Text",
  Evaluatable->False],
Cell[14695, 409, 86, 4, 84, "Input"],
Cell[14784, 415, 149, 5, 41, "Text",
  Evaluatable->False],
Cell[14936, 422, 116, 2, 45, "Input"],
Cell[15055, 426, 110, 2, 41, "Text",
  Evaluatable->False],
Cell[15168, 430, 98, 2, 45, "Input"],
Cell[15269, 434, 156, 5, 41, "Text",
  Evaluatable->False],
Cell[15428, 441, 173, 5, 68, "Input"],
Cell[15604, 448, 154, 5, 41, "Text",
  Evaluatable->False],
Cell[15761, 455, 211, 5, 91, "Input"],
Cell[15975, 462, 311, 7, 79, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[16323, 474, 122, 3, 64, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:8"],
Cell[16448, 479, 43, 0, 44, "Input"],
Cell[16494, 481, 172, 4, 91, "Input"],
Cell[16669, 487, 174, 5, 41, "Text",
  Evaluatable->False],
Cell[16846, 494, 104, 3, 68, "Input"],
Cell[16953, 499, 93, 2, 45, "Input"],
Cell[17049, 503, 146, 2, 68, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17232, 510, 134, 3, 64, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:9"],
Cell[17369, 515, 387, 8, 79, "Text",
  Evaluatable->False],
Cell[17759, 525, 72, 0, 44, "Input"],
Cell[17834, 527, 105, 2, 45, "Input"],
Cell[17942, 531, 78, 2, 45, "Input"],
Cell[18023, 535, 234, 6, 60, "Text",
  Evaluatable->False],
Cell[18260, 543, 106, 2, 45, "Input"],
Cell[18369, 547, 77, 2, 45, "Input"],
Cell[18449, 551, 133, 3, 68, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18619, 559, 128, 3, 64, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:10"],
Cell[18750, 564, 334, 7, 79, "Text",
  Evaluatable->False],
Cell[19087, 573, 184, 4, 91, "Input"],
Cell[19274, 579, 145, 5, 41, "Text",
  Evaluatable->False],
Cell[19422, 586, 125, 4, 84, "Input"],
Cell[19550, 592, 94, 2, 41, "Text",
  Evaluatable->False],
Cell[19647, 596, 93, 2, 45, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[19777, 603, 118, 3, 64, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:11"],
Cell[19898, 608, 220, 6, 60, "Text",
  Evaluatable->False],
Cell[20121, 616, 80, 3, 64, "Input"],
Cell[20204, 621, 202, 4, 91, "Input"],
Cell[20409, 627, 326, 8, 122, "Text",
  Evaluatable->False],
Cell[20738, 637, 62, 3, 64, "Input"],
Cell[20803, 642, 202, 4, 91, "Input"],
Cell[21008, 648, 123, 5, 72, "Text",
  Evaluatable->False],
Cell[21134, 655, 127, 3, 68, "Input"],
Cell[21264, 660, 202, 4, 91, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[21527, 671, 155, 4, 66, "Section",
  Evaluatable->False,
  CellTags->"c:12"],

Cell[CellGroupData[{
Cell[21707, 679, 159, 4, 57, "Subsection",
  Evaluatable->False,
  CellTags->"c:13"],
Cell[21869, 685, 694, 12, 136, "Text",
  Evaluatable->False],
Cell[22566, 699, 157, 4, 68, "Input"],
Cell[22726, 705, 89, 2, 45, "Input"],
Cell[22818, 709, 245, 6, 60, "Text",
  Evaluatable->False],
Cell[23066, 717, 98, 2, 45, "Input"],
Cell[23167, 721, 137, 5, 41, "Text",
  Evaluatable->False],
Cell[23307, 728, 140, 4, 91, "Input"],
Cell[23450, 734, 160, 5, 41, "Text",
  Evaluatable->False],
Cell[23613, 741, 120, 3, 45, "Input"],
Cell[23736, 746, 115, 2, 41, "Text",
  Evaluatable->False],
Cell[23854, 750, 88, 2, 45, "Input"],
Cell[23945, 754, 260, 7, 91, "Text",
  Evaluatable->False],
Cell[24208, 763, 231, 6, 137, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[24476, 774, 159, 4, 46, "Subsection",
  Evaluatable->False,
  CellTags->"c:14"],
Cell[24638, 780, 766, 15, 198, "Text",
  Evaluatable->False],
Cell[25407, 797, 313, 7, 137, "Input"],
Cell[25723, 806, 87, 2, 45, "Input"],
Cell[25813, 810, 87, 2, 45, "Input"],
Cell[25903, 814, 74, 2, 45, "Input"],
Cell[25980, 818, 83, 2, 45, "Input"],
Cell[26066, 822, 74, 2, 45, "Input"],
Cell[26143, 826, 309, 7, 79, "Text",
  Evaluatable->False],
Cell[26455, 835, 206, 4, 68, "Input"],
Cell[26664, 841, 110, 2, 45, "Input"],
Cell[26777, 845, 80, 2, 45, "Input"],
Cell[26860, 849, 129, 5, 72, "Text",
  Evaluatable->False],
Cell[26992, 856, 209, 4, 91, "Input"],
Cell[27204, 862, 74, 2, 45, "Input"],
Cell[27281, 866, 84, 2, 45, "Input"],
Cell[27368, 870, 84, 2, 45, "Input"],
Cell[27455, 874, 75, 2, 45, "Input"],
Cell[27533, 878, 84, 2, 45, "Input"],
Cell[27620, 882, 75, 2, 45, "Input"],
Cell[27698, 886, 155, 4, 45, "Input"],
Cell[27856, 892, 79, 2, 45, "Input"],
Cell[27938, 896, 490, 11, 191, "Text",
  Evaluatable->False],
Cell[28431, 909, 185, 4, 91, "Input"],
Cell[28619, 915, 74, 2, 45, "Input"],
Cell[28696, 919, 84, 2, 45, "Input"],
Cell[28783, 923, 84, 2, 45, "Input"],
Cell[28870, 927, 75, 2, 45, "Input"],
Cell[28948, 931, 84, 2, 45, "Input"],
Cell[29035, 935, 75, 2, 45, "Input"],
Cell[29113, 939, 155, 4, 45, "Input"],
Cell[29271, 945, 352, 9, 103, "Text",
  Evaluatable->False]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

